#define PERL_constant_NOTFOUND	1
#define PERL_constant_NOTDEF	2
#define PERL_constant_ISIV	3
#define PERL_constant_ISNO	4
#define PERL_constant_ISNV	5
#define PERL_constant_ISPV	6
#define PERL_constant_ISPVN	7
#define PERL_constant_ISSV	8
#define PERL_constant_ISUNDEF	9
#define PERL_constant_ISUV	10
#define PERL_constant_ISYES	11

#ifndef NVTYPE
typedef double NV; /* 5.6 and later define NVTYPE, and typedef NV to it.  */
#endif
#ifndef aTHX_
#define aTHX_ /* 5.6 or later define this for threading support.  */
#endif
#ifndef pTHX_
#define pTHX_ /* 5.6 or later define this for threading support.  */
#endif

static void
constant_add_symbol(pTHX_  HV *hash, const char *name, I32 namelen, SV *value) {
    HE *he = (HE*) hv_common_key_len(hash, name, namelen, HV_FETCH_LVALUE, NULL,
				     0);
    SV *sv;

    if (!he) {
        croak("Couldn't add key '%s' to %%Socket::Linux::",
		   name);
    }
    sv = HeVAL(he);
    if (SvOK(sv) || SvTYPE(sv) == SVt_PVGV) {
	/* Someone has been here before us - have to make a real sub.  */
	newCONSTSUB(hash, name, value);
    } else {
	SvUPGRADE(sv, SVt_RV);
	SvRV_set(sv, value);
	SvROK_on(sv);
	SvREADONLY_on(value);
    }
}



#ifndef SYMBIAN

/* Store a hash of all symbols missing from the package. To avoid trampling on
   the package namespace (uninvited) put each package's hash in our namespace.
   To avoid creating lots of typeblogs and symbol tables for sub-packages, put
   each package's hash into one hash in our namespace.  */

static HV *
get_missing_hash(pTHX) {
    HV *const parent
	= get_hv("ExtUtils::Constant::ProxySubs::Missing", GVf_MULTI);
    /* We could make a hash of hashes directly, but this would confuse anything
	at Perl space that looks at us, and as we're visible in Perl space,
	best to play nice. */
    SV *const *const ref
	= hv_fetch(parent, "Socket::Linux", 13, TRUE);
    HV *new_hv;

    if (!ref)
	return NULL;

    if (SvROK(*ref))
	return (HV*) SvRV(*ref);

    new_hv = newHV();
    SvUPGRADE(*ref, SVt_RV);
    SvRV_set(*ref, (SV *)new_hv);
    SvROK_on(*ref);
    return new_hv;
}

#endif

struct notfound_s {const char *name; I32 namelen;} ;

    static const struct notfound_s values_for_notfound[] =
      {
#ifndef TCP_NODELAY
        { "TCP_NODELAY", 11 },
#endif
#ifndef TCP_MAXSEG
        { "TCP_MAXSEG", 10 },
#endif
#ifndef TCP_CORK
        { "TCP_CORK", 8 },
#endif
#ifndef TCP_KEEPIDLE
        { "TCP_KEEPIDLE", 12 },
#endif
#ifndef TCP_KEEPINTVL
        { "TCP_KEEPINTVL", 13 },
#endif
#ifndef TCP_KEEPCNT
        { "TCP_KEEPCNT", 11 },
#endif
#ifndef TCP_SYNCNT
        { "TCP_SYNCNT", 10 },
#endif
#ifndef TCP_LINGER2
        { "TCP_LINGER2", 11 },
#endif
#ifndef TCP_DEFER_ACCEPT
        { "TCP_DEFER_ACCEPT", 16 },
#endif
#ifndef TCP_WINDOW_CLAMP
        { "TCP_WINDOW_CLAMP", 16 },
#endif
#ifndef TCP_INFO
        { "TCP_INFO", 8 },
#endif
#ifndef TCP_QUICKACK
        { "TCP_QUICKACK", 12 },
#endif
#ifndef TCP_CONGESTION
        { "TCP_CONGESTION", 14 },
#endif
#ifndef TCP_MD5SIG
        { "TCP_MD5SIG", 10 },
#endif
#ifndef TCPOPT_EOL
        { "TCPOPT_EOL", 10 },
#endif
#ifndef TCPOPT_NOP
        { "TCPOPT_NOP", 10 },
#endif
#ifndef TCPOPT_MAXSEG
        { "TCPOPT_MAXSEG", 13 },
#endif
#ifndef TCPOLEN_MAXSEG
        { "TCPOLEN_MAXSEG", 14 },
#endif
#ifndef TCPOPT_WINDOW
        { "TCPOPT_WINDOW", 13 },
#endif
#ifndef TCPOLEN_WINDOW
        { "TCPOLEN_WINDOW", 14 },
#endif
#ifndef TCPOPT_SACK_PERMITTED
        { "TCPOPT_SACK_PERMITTED", 21 },
#endif
#ifndef TCPOLEN_SACK_PERMITTED
        { "TCPOLEN_SACK_PERMITTED", 22 },
#endif
#ifndef TCPOPT_SACK
        { "TCPOPT_SACK", 11 },
#endif
#ifndef TCPOPT_TIMESTAMP
        { "TCPOPT_TIMESTAMP", 16 },
#endif
#ifndef TCPOLEN_TIMESTAMP
        { "TCPOLEN_TIMESTAMP", 17 },
#endif
#ifndef TCPOLEN_TSTAMP_APPA
        { "TCPOLEN_TSTAMP_APPA", 19 },
#endif
#ifndef TCPOPT_TSTAMP_HDR
        { "TCPOPT_TSTAMP_HDR", 17 },
#endif
#ifndef TCP_MSS
        { "TCP_MSS", 7 },
#endif
#ifndef TCP_MAXWIN
        { "TCP_MAXWIN", 10 },
#endif
#ifndef TCP_MAX_WINSHIFT
        { "TCP_MAX_WINSHIFT", 16 },
#endif
#ifndef SOL_TCP
        { "SOL_TCP", 7 },
#endif
#ifndef TCPI_OPT_TIMESTAMPS
        { "TCPI_OPT_TIMESTAMPS", 19 },
#endif
#ifndef TCPI_OPT_SACK
        { "TCPI_OPT_SACK", 13 },
#endif
#ifndef TCPI_OPT_WSCALE
        { "TCPI_OPT_WSCALE", 15 },
#endif
#ifndef TCPI_OPT_ECN
        { "TCPI_OPT_ECN", 12 },
#endif
#ifndef TCP_MD5SIG_MAXKEYLEN
        { "TCP_MD5SIG_MAXKEYLEN", 20 },
#endif
        { NULL, 0 } };
struct iv_s {const char *name; I32 namelen; IV value;};
